@@ -39,8 +39,9 @@ else:
 from RNS.vendor.configobj import ConfigObj
 import configparser
 import multiprocessing.connection
-import signal
+import importlib
 import threading
+import signal
 import atexit
 import struct
 import array
@@ -89,7 +90,7 @@ class Reticulum:
     the default value.
     """
 
-    LINK_MTU_DISCOVERY   = False
+    LINK_MTU_DISCOVERY   = True
     """
     Whether automatic link MTU discovery is enabled by default in this
     release. Link MTU discovery significantly increases throughput over
@@ -163,19 +164,23 @@ class Reticulum:
     __instance       = None
 
     __interface_detach_ran = False
+    __exit_handler_ran = False
     @staticmethod
     def exit_handler():
         # This exit handler is called whenever Reticulum is asked to
         # shut down, and will in turn call exit handlers in other
         # classes, saving necessary information to disk and carrying
         # out cleanup operations.
-        if not Reticulum.__interface_detach_ran:
-            RNS.Transport.detach_interfaces()
-        RNS.Transport.exit_handler()
-        RNS.Identity.exit_handler()
+        if not Reticulum.__exit_handler_ran:
+            if not Reticulum.__interface_detach_ran:
+                RNS.Transport.detach_interfaces()
+            RNS.Transport.exit_handler()
+            RNS.Identity.exit_handler()
 
-        if RNS.Profiler.ran():
-            RNS.Profiler.results()
+            if RNS.Profiler.ran():
+                RNS.Profiler.results()
+
+            RNS.loglevel = -1
 
     @staticmethod
     def sigint_handler(signal, frame):
@@ -300,7 +305,7 @@ class Reticulum:
             time.sleep(1.5)
 
         self.__apply_config()
-        RNS.log(f"Utilising cryptography backend \"{RNS.Cryptography.Provider.backend()}\"", RNS.LOG_VERBOSE)
+        RNS.log(f"Utilising cryptography backend \"{RNS.Cryptography.Provider.backend()}\"", RNS.LOG_DEBUG)
         RNS.log(f"Configuration loaded from {self.configpath}", RNS.LOG_VERBOSE)
         
         RNS.Identity.load_known_destinations()
@@ -479,6 +484,8 @@ class Reticulum:
                     if v == False:
                         Reticulum.__use_implicit_proof = False
 
+        if RNS.compiled: RNS.log("Reticulum running in compiled mode", RNS.LOG_DEBUG)
+        else: RNS.log("Reticulum running in interpreted mode", RNS.LOG_DEBUG)
         self.__start_local_interface()
 
         if self.is_shared_instance or self.is_standalone_instance:
@@ -643,6 +650,7 @@ class Reticulum:
                                         interface.ifac_signature = interface.ifac_identity.sign(RNS.Identity.full_hash(interface.ifac_key))
 
                                     RNS.Transport.interfaces.append(interface)
+                                    interface.final_init()
 
                             interface = None
                             if (("interface_enabled" in c) and c.as_bool("interface_enabled") == True) or (("enabled" in c) and c.as_bool("enabled") == True):
@@ -800,6 +808,7 @@ class Reticulum:
                     interface.ifac_signature = interface.ifac_identity.sign(RNS.Identity.full_hash(interface.ifac_key))
 
                 RNS.Transport.interfaces.append(interface)
+                interface.final_init()
 
     def _should_persist_data(self):
         if time.time() > self.last_data_persist+Reticulum.GRACIOUS_PERSIST_INTERVAL:
@@ -983,12 +992,16 @@ class Reticulum:
                         ifstats["rxs"] = interface.current_rx_speed
                     else:
                         ifstats["rxs"] = 0
+                else:
+                    ifstats["rxs"] = 0
 
                 if hasattr(interface, "current_tx_speed"):
                     if interface.current_tx_speed != None:
                         ifstats["txs"] = interface.current_tx_speed
                     else:
                         ifstats["txs"] = 0
+                else:
+                    ifstats["txs"] = 0
 
                 if hasattr(interface, "peers"):
                     if interface.peers != None:
@@ -1039,6 +1052,13 @@ class Reticulum:
                 else:
                     stats["probe_responder"] = None
 
+            if importlib.util.find_spec('psutil') != None:
+                import psutil
+                process = psutil.Process()
+                stats["rss"] = process.memory_info().rss
+            else:
+                stats["rss"] = None
+
             return stats
 
     def get_path_table(self, max_hops=None):
