@@ -49,6 +49,11 @@ pyc_modules = glob.glob(os.path.dirname(__file__)+"/*.pyc")
 modules     = py_modules+pyc_modules
 __all__ = list(set([os.path.basename(f).replace(".pyc", "").replace(".py", "") for f in modules if not (f.endswith("__init__.py") or f.endswith("__init__.pyc"))]))
 
+import importlib
+if importlib.util.find_spec("cython"): import cython; compiled = cython.compiled
+else: compiled = False
+
+LOG_NONE     = -1
 LOG_CRITICAL = 0
 LOG_ERROR    = 1
 LOG_WARNING  = 2
@@ -81,21 +86,21 @@ logging_lock = threading.Lock()
 
 def loglevelname(level):
     if (level == LOG_CRITICAL):
-        return "Critical"
+        return "[Critical]"
     if (level == LOG_ERROR):
-        return "Error"
+        return "[Error]   "
     if (level == LOG_WARNING):
-        return "Warning"
+        return "[Warning] "
     if (level == LOG_NOTICE):
-        return "Notice"
+        return "[Notice]  "
     if (level == LOG_INFO):
-        return "Info"
+        return "[Info]    "
     if (level == LOG_VERBOSE):
-        return "Verbose"
+        return "[Verbose] "
     if (level == LOG_DEBUG):
-        return "Debug"
+        return "[Debug]   "
     if (level == LOG_EXTREME):
-        return "Extra"
+        return "[Extra]   "
     
     return "Unknown"
 
@@ -114,20 +119,22 @@ def precise_timestamp_str(time_s):
     return datetime.datetime.now().strftime(logtimefmt_p)[:-3]
 
 def log(msg, level=3, _override_destination = False, pt=False):
+    if loglevel == LOG_NONE: return
     global _always_override_destination, compact_log_fmt
     msg = str(msg)
     if loglevel >= level:
         if pt:
-            logstring = "["+precise_timestamp_str(time.time())+"] ["+loglevelname(level)+"] "+msg
+            logstring = "["+precise_timestamp_str(time.time())+"] "+loglevelname(level)+" "+msg
         else:
             if not compact_log_fmt:
-                logstring = "["+timestamp_str(time.time())+"] ["+loglevelname(level)+"] "+msg
+                logstring = "["+timestamp_str(time.time())+"] "+loglevelname(level)+" "+msg
             else:
-                logstring = "["+timestamp_str(time.time())+"] "+msg
+                logstring = "["+timestamp_str(time.time())+" "+msg
 
         with logging_lock:
             if (logdest == LOG_STDOUT or _always_override_destination or _override_destination):
-                print(logstring)
+                if not threading.main_thread().is_alive(): return
+                else: print(logstring)
 
             elif (logdest == LOG_FILE and logfile != None):
                 try:
@@ -361,9 +368,14 @@ def phyparams():
 def panic():
     os._exit(255)
 
-def exit():
-    print("")
-    sys.exit(0)
+exit_called = False
+def exit(code=0):
+    global exit_called
+    if not exit_called:
+        exit_called = True
+        print("")
+        Reticulum.exit_handler()
+        os._exit(code)
 
 class Profiler:
     _ran = False
